#!/usr/bin/env python
#
# PkgTrackInst.PlistWriter - Writes pkg-plist based on install log
#
# ----------------------------------------------------------------------------
# "THE BEER-WARE LICENSE" (Revision 42, (c) Poul-Henning Kamp):
# Hye-Shik Chang <perky@FreeBSD.org> wrote this file.  As long as you retain
# this notice you can do whatever you want with this stuff. If we meet some
# day, and you think this stuff is worth it, you can buy me a beer in return.
#
# Hye-Shik Chang
# ----------------------------------------------------------------------------
# $Id: PlistWriter.py 482 2005-05-03 14:18:20Z perky $

__pychecker__ = 'no-argsused no-isliteral'

import os, sys
import time, re
from PkgTrackInst import options
from PkgTrackInst.cmd_support import errx, echo_msg
from PkgTrackInst.Trackers import InstFile, InstDirectory, TrackRoot

SUBST_IGNORE = ('PREFIX', 'PORTDOCS', 'LOCALBASE', 'X11BASE')
MANPAGELIST_FILENAME = 'MANPAGES_LIST'
MANPAGELIST_LINEWIDTH = 64


class PlistWriter(object):

    def __init__(self):
        self.plist_subst = []
        self.prefix = self.get_make_env('PREFIX')
        if options.substitute:
            self.build_sub_list()

    def get_output(self, cmd):
        f = os.popen(cmd, 'r')
        r = f.read()
        status = f.close()
        if status:
            print repr(status)
            errx("command `%s' exited with error code %d" % (cmd, status))
        return r

    def get_make_env(self, varname):
        return self.get_output('make -V ' + varname).strip()

    def build_sub_list(self):
        pat_subst = re.compile('([A-Za-z_0-9]+)=("([^"]*)"|[^"]\S*)')
        substlist = self.get_make_env('PLIST_SUB')

        for var, raw, stripped in pat_subst.findall(substlist):
            if var not in SUBST_IGNORE:
                self.plist_subst.append((stripped or raw, var))
        self.plist_subst.sort(reverse=True)

    def write(self):
        if os.path.exists('pkg-plist'):
            os.rename('pkg-plist', 'pkg-plist.bak')
        self.clear_manpath()
        self.generate_plist(open('pkg-plist', 'w'))

    def relpath(self, fullpath):
        prefix = self.prefix.split(os.sep)
        full = fullpath.split(os.sep)
        if full[:len(prefix)] == prefix:
            repled = os.sep.join(full[len(prefix):])
            for orig, subst in self.plist_subst:
                if repled.startswith(orig+'/'):
                    return '%%' + subst + '%%' + repled[len(orig):]
            return repled
        else:
            return fullpath

    def clear_manpath(self):
        pat_manfile = re.compile('([A-Za-z_0-9]+)\.([0-9ln])(\.gz)?')
        manpages = {}

        for category in '123456789ln':
            pages = []
            for ftype in ('man', 'cat'):
                path = os.sep.join((self.prefix, 'man', ftype+category))

                node = TrackRoot[path]
                if not isinstance(node, InstDirectory):
                    errx("`%s' is not a directory." % path)

                for name, child in node.iterchildren():
                    if isinstance(child, InstDirectory):
                        continue # ignore subdirectories in man path.

                    m = pat_manfile.match(name)
                    if not m:
                        continue # ignore files with non-uniform name

                    m_subject, m_category, m_suffix = m.groups()
                    if category != m_category:
                        errx("`%s' is miscategorized in the installation" %
                             path)
                    pages.append(m_subject)
                    child.created = False

            if pages:
                manpages[category] = pages

        if manpages:
            echo_msg("Writing manpages list as `%s'" % MANPAGELIST_FILENAME)

            f = open(MANPAGELIST_FILENAME, 'w')
            for cate, pages in manpages.iteritems():
                out = ['']
                for page in sorted(pages):
                    item = page + '.' + cate
                    if (len(item) + len(out[-1]) > MANPAGELIST_LINEWIDTH and
                            out[-1]): # put 1 at least
                        out[-1] += ' \\'
                        out.append('')
                    out[-1] += ' ' + item

                print >> f, 'MAN%s=\t%s' % (cate.upper(), out[0][1:])
                for line in out[1:]:
                    print >> f, '\t%s' % line[1:]

    def generate_plist(self, out):
        echo_msg('Generating pkg-plist')

        fileents, dirents = [], []
        def gather_entries(ent):
            if ent.created:
                if isinstance(ent, InstDirectory):
                    dirents.append(ent)
                else:
                    fileents.append(ent)
        TrackRoot.walk(gather_entries)

        fileents.sort(key=lambda ent: ent.abspath)
        dirents.sort(key=lambda ent: ent.abspath)

        for ent in fileents:
            print >> out, self.relpath(ent.abspath)
        for ent in reversed(dirents):
            print >> out, '@dirrm ' + self.relpath(ent.abspath)
